--
-- HawePipe
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2015-01-01 - Initial implementation
-- 

HawePipe = {};

function HawePipe.prerequisitesPresent(specializations)
    return true;
end;

-- add check for if pipe is being used.
function HawePipe:allowAdding(vehicle)
	if vehicle.pipeIsUnloading and vehicle.currentPipeParticleSystem ~= nil then
		local txt;
		
		if g_i18n:hasText("ERROR_PIPE_IS_BEING_USED") then
			txt = g_i18n:getText("ERROR_PIPE_IS_BEING_USED");
		end;
		
		return false, txt;
	end;
	
	return true;
end;

function HawePipe:allowRemoval(vehicle)
	local allow, txt = HawePipe.allowAdding(nil, vehicle);
	return allow, txt;
end;

function HawePipe:load(xmlFile)
	self.savedData = {};
	
	HawePipe.fillVolumeData(xmlFile, self, "fillVolumeDischargeInfo");
	
	local pipeRaycastNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.pipe#raycastNodeIndex"));
	if pipeRaycastNode ~= nil then
		self.oldPipe = self.vehicle.pipeRaycastNode;
		self.vehicle.pipeRaycastNode = pipeRaycastNode;
	end;
	
	if self.isClient then
		self.pipeEffect = EffectManager:loadEffect(xmlFile, "vehicle.pipeEffect", self.components, self);
		
		if self.pipeEffect ~= nil then
			self.savedPipeEffect = self.vehicle.pipeEffect;
			self.vehicle.pipeEffect = self.pipeEffect;
		end;
	end;
	
	if self.controlUnitVersion <= 1.31 then -- new feature of HP 1.21
		self.skinnedNodes = {};
		local i = 0;
		while true do
			local key = string.format("vehicle.skinnedNodes.skinnedNode(%d)", i);
			if not hasXMLProperty(xmlFile, key) then break; end;
			
			local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key .. "#index"));
			if node ~= nil then
				if Utils.getNoNil(getXMLBool(xmlFile, key .. "#moveToZero"), false) then
					setTranslation(node, 0,0,0);
					setRotation(node, 0,0,0);
				end;
				
				table.insert(self.skinnedNodes, node);
			end;
			i = i + 1;
		end;
		
		for _, node in pairs(self.skinnedNodes) do
			local minClipDistance = getClipDistance(node);
			local curNode = getParent(node);
			while curNode ~= 0 do
				minClipDistance = math.min(minClipDistance, getClipDistance(curNode));
				curNode = getParent(curNode);
			end
			setClipDistance(node, minClipDistance)

			link(getRootNode(), node);
		end;
	end;
	
	if self.controlUnitVersion <= 1.32 then -- new feature of HP 1.22
		if self.getOwner == nil then
			self.getOwner = HawePipe.getOwner;
		end;
	end;
end;

function HawePipe.fillVolumeData(xmlFile, self, name)
	local entry = {};
	entry.name = name;
	
	local indexString = getXMLString(xmlFile, "vehicle." .. entry.name .. "#index");
	if indexString == nil then
		entry.node = createTransformGroup(entry.name);
		link(self.components[1].node, entry.node);
		setTranslation(entry.node, 0, 0, 0);
	else
		entry.node = Utils.indexToObject(self.components, indexString);
	end;
	
	entry.width = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle." .. entry.name .. "#width"), 1.0);
	entry.length = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle." .. entry.name .. "#length"), 1.0);

	if entry.name == "fillVolumeDischargeInfo" then
		entry.alsoUseLoadInfoForDischarge = Utils.getNoNil(getXMLBool(xmlFile, "vehicle." .. entry.name .. "#alsoUseLoadInfoForDischarge"), false);
		entry.loadInfoSizeScale = Utils.getVectorNFromString( Utils.getNoNil(getXMLString(xmlFile, "vehicle." .. entry.name .. "#loadInfoSizeScale"), "1 1"), 2 );
		entry.loadInfoFillFactor = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle." .. entry.name .. "#loadInfoFillFactor"), 0.6);
	end;
	
	if entry.node ~= nil then
		self.savedData[entry.name] = self.vehicle[entry.name];
		self.vehicle[entry.name] = entry;
	end;
end;

function HawePipe:delete()
	for name, v in pairs(self.savedData) do
		self.vehicle[name] = self.savedData[name];
	end;
	
	if self.oldPipe ~= nil then
		self.vehicle.pipeRaycastNode = self.oldPipe;
	end;
	
	if self.isClient then
		EffectManager:deleteEffect(self.pipeEffect);
		self.vehicle.pipeEffect = self.savedPipeEffect;
	end;
	
	if self.controlUnitVersion <= 1.31 then
		for _, node in pairs(self.skinnedNodes) do
			delete(node);
		end;
    end;
end;

function HawePipe:readStream(streamId, connection)
end;

function HawePipe:writeStream(streamId, connection)
end;

function HawePipe:update(dt)
end;

function HawePipe:updateTick(dt)
end;

function HawePipe:draw()
end;

-- fix for cylinder spec with older Hardpoint versions then v1.22
function HawePipe:getOwner()
	return self.vehicle:getOwner();
end;